package com.unisystems.ha;

import java.util.List;

import org.apache.log4j.Logger;
import org.jboss.soa.esb.addressing.EPR;
import org.jboss.soa.esb.listeners.ha.AbstractLoadBalancePolicy;
import org.jboss.soa.esb.listeners.ha.RandomRobin;
import org.jboss.soa.esb.listeners.ha.ServiceClusterInfo;

public class LocalPreferenceBalancer extends AbstractLoadBalancePolicy {
	   
	protected Logger _logger = Logger.getLogger(this.getClass().getName());
	protected transient EPR electedEPR = null;
	
	@Override
	protected EPR chooseEPRByPolicy(ServiceClusterInfo serviceCluster) {

		List<EPR> eprs = serviceCluster.getEPRs();
		
	    if (eprs.size () == 0)
	         return null;
	    
	    if ( (this.electedEPR != null) && eprs.contains (this.electedEPR) ){
	         return this.electedEPR;
	      } else {
	    	  boolean found = false;
	    	  for (EPR epr : eprs) {
	    		  String eprUrl = epr.getAddr().getExtensionValue("java.naming.provider.url");
	    		  _logger.debug("eprUrl " + eprUrl);
	    		  String serverProviderUrl =System.getProperty("java.naming.provider.url"); 
	    		  _logger.debug("serverProviderUrl " + serverProviderUrl);
	    		  if(serverProviderUrl.endsWith(eprUrl)){
	    			  _logger.debug("selected " + epr.getAddr().getAddress());
	    			  this.electedEPR = epr;
	    			  found = true;
	    			  break;
	    		  }
	    	  }
	    	  
	    	  if(!found){
	    		  _logger.debug("No local EPR found choosing EPR with round robin");
    	         int cursor = RandomRobin.localRandomizer.nextInt(eprs.size());
    	         this.electedEPR = eprs.get(cursor);
	    	  }
	         return this.electedEPR;
	      }


	}

}
